@@ -55,6 +55,7 @@
 #include <errno.h>
 #include "nilfs.h"
 #include "parser.h"
+#include "util.h"
 
 
 #ifdef _GNU_SOURCE
@@ -106,7 +107,7 @@ static int rmcp_remove_range(struct nilfs *nilfs,
 	int ret = 0;
 
 	for (cno = start; cno <= end; cno++) {
-		if (nilfs_delete_checkpoint(nilfs, cno) == 0) {
+		if (likely(nilfs_delete_checkpoint(nilfs, cno) == 0)) {
 			nd++;
 			continue;
 		}
@@ -146,7 +147,6 @@ int main(int argc, char *argv[])
 	int option_index;
 #endif	/* _GNU_SOURCE */
 
-	opterr = 0;
 	last = strrchr(argv[0], '/');
 	progname = last ? last + 1 : argv[0];
 
@@ -174,7 +174,7 @@ int main(int argc, char *argv[])
 			       PACKAGE_VERSION);
 			exit(EXIT_SUCCESS);
 		default:
-			errx(EXIT_FAILURE, "invalid option -- %c", optopt);
+			exit(EXIT_FAILURE);
 		}
 	}
 
@@ -194,7 +194,8 @@ int main(int argc, char *argv[])
 	if (nilfs == NULL)
 		err(EXIT_FAILURE, "cannot open NILFS on %s", dev ? : "device");
 
-	if (nilfs_get_cpstat(nilfs, &cpstat) < 0) {
+	ret = nilfs_get_cpstat(nilfs, &cpstat);
+	if (unlikely(ret < 0)) {
 		warn("%s: cannot get checkpoint status", dev);
 		status = EXIT_FAILURE;
 		goto out_close_nilfs;
