@@ -71,12 +71,15 @@
 #endif	/* HAVE_BLKID_BLKID_H */
 
 #include "nilfs.h"
+#include "compat.h"
 #include "util.h"
 #include "nilfs_feature.h"
 #include "mkfs.h"
 #include "pathnames.h"
 
 
+extern int check_mount(const char *device);
+
 typedef __u64  blocknr_t;
 
 extern __u32 crc32_le(__u32 seed, unsigned char const *data, size_t length);
@@ -152,7 +155,7 @@ static void parse_options(int argc, char *argv[]);
 static const unsigned group_desc_blocks_per_group = 1;
 static const unsigned bitmap_blocks_per_group = 1;
 static const unsigned nr_initial_segments = 2; /* initial segment + next */
-static const unsigned nr_initial_inodes = 2;  /* root directory + .nilfs */
+static const unsigned nr_initial_inodes = 1;  /* root directory */
 static const __u64 first_cno = 1; /* Number of the first checkpoint */
 
 /* Segment layout information (per partial segment) */
@@ -268,7 +271,6 @@ static void init_nilfs(struct nilfs_disk_info *di);
 static void prepare_segment(struct nilfs_segment_info *);
 static void commit_segment(void);
 static void nilfs_mkfs_make_rootdir(void);
-static void nilfs_mkfs_make_dot_nilfs(void);
 static void nilfs_mkfs_make_reserved_files(void);
 
 
@@ -338,7 +340,6 @@ static int nilfs_mkfs_discard_zeroes_data(int fd)
 #endif
 
 static void disk_scan(const char *device);
-static void check_mount(int fd, const char *device);
 
 #if HAVE_LIBBLKID
 static void check_safety_of_device_overwrite(int fd, const char *device);
@@ -594,7 +595,7 @@ int main(int argc, char *argv[])
 	struct nilfs_segment_info *si;
 	struct stat statbuf;
 	const char *device;
-	int fd;
+	int fd, ret;
 
 	parse_options(argc, argv);
 	device = argv[optind];
@@ -607,17 +608,25 @@ int main(int argc, char *argv[])
 	if (cflag)
 		disk_scan(device);  /* check the block device */
 
+	ret = check_mount(device);
+	if (ret < 0)
+		perr("Error checking mount status of %s: %s", device,
+		     strerror(errno));
+	if (ret > 0)
+		perr("Error: %s is currently mounted. You cannot make a filesystem on this device.",
+		     device);
+
 	fd = open(device, O_RDWR);
 	if (fd < 0)
-		perr("Error: cannot open device: %s", device);
-	check_mount(fd, device);
+		perr("Error: cannot open device %s: %s", device,
+		     strerror(errno));
+
 	check_safety_of_device_overwrite(fd, device);
 
 	init_disk_layout(di, fd, device);
 	si = new_segment(di);
 
 	add_file(si, NILFS_ROOT_INO, 1, 0);
-	add_file(si, NILFS_NILFS_INO, 0, 0);
 	add_file(si, NILFS_ATIME_INO, 0, 0);
 	add_file(si, 1, 0, 0);
 	add_file(si, 8, 0, 0);
@@ -638,7 +647,6 @@ int main(int argc, char *argv[])
 	init_nilfs(di);
 
 	prepare_segment(&di->seginfo[0]);
-	nilfs_mkfs_make_dot_nilfs(); /* Make .nilfs */
 	nilfs_mkfs_make_rootdir();  /* Make root directory */
 	nilfs_mkfs_make_reserved_files();
 	commit_segment();
@@ -707,28 +715,6 @@ static void disk_scan(const char *device)
 	}
 }
 
-static void check_mount(int fd, const char *device)
-{
-	FILE *fp;
-	char line[LINE_BUFFER_SIZE];
-
-	fp = fopen(_PATH_MOUNTED, "r");
-	if (fp == NULL) {
-		close(fd);
-		perr("Error: cannot open %s!", _PATH_MOUNTED);
-	}
-
-	while (fgets(line, LINE_BUFFER_SIZE, fp) != NULL) {
-		if (strncmp(strtok(line, " "), device, strlen(device)) == 0) {
-			fclose(fp);
-			close(fd);
-			perr("Error: %s is currently mounted. You cannot make a filesystem on this device.",
-			     device);
-		}
-	}
-	fclose(fp);
-}
-
 #if HAVE_LIBBLKID
 static void check_safety_of_device_overwrite(int fd, const char *device)
 {
@@ -1288,26 +1274,13 @@ static void nilfs_mkfs_make_rootdir(void)
 	de = next_dir_entry(de);
 	de->inode = cpu_to_le64(NILFS_ROOT_INO);
 	de->name_len = 2;
-	rec_end += (rec_len = NILFS_DIR_REC_LEN(2));
-	de->rec_len = nilfs_rec_len_to_disk(rec_len);
+	de->rec_len = nilfs_rec_len_to_disk(blocksize - rec_end);
 	de->file_type = NILFS_FT_DIR;
 	memcpy((void *)de->name, "..\0\0\0\0\0", 8);
 
-	de = next_dir_entry(de);
-	de->inode = cpu_to_le64(NILFS_NILFS_INO);
-	de->name_len = 6;
-	de->rec_len = nilfs_rec_len_to_disk(blocksize - rec_end);
-	de->file_type = NILFS_FT_REG_FILE;
-	memcpy((void *)de->name, ".nilfs\0", 8);
-
 	inc_link_count(NILFS_ROOT_INO);
 }
 
-static void nilfs_mkfs_make_dot_nilfs(void)
-{
-	init_inode(NILFS_NILFS_INO, DT_REG, 0644, 0);
-}
-
 static void nilfs_mkfs_make_reserved_files(void)
 {
 	init_inode(NILFS_ATIME_INO, DT_REG, 0, 0);
@@ -1638,7 +1611,7 @@ static void prepare_segment(struct nilfs_segment_info *si)
 	/* initialize super root */
 	nilfs.super_root = map_disk_buffer(si->start + si->nblocks - 1, 1);
 	sr = nilfs.super_root;
-	sr->sr_bytes = cpu_to_le16(NILFS_SR_BYTES);
+	sr->sr_bytes = cpu_to_le16(NILFS_SR_BYTES(sizeof(struct nilfs_inode)));
 	sr->sr_nongc_ctime = cpu_to_le64(di->ctime);
 	sr->sr_flags = 0;
 
@@ -1657,6 +1630,7 @@ static void fill_in_checksums(struct nilfs_segment_info *si, __u32 crc_seed)
 	blocknr_t blocknr;
 	unsigned long rest_blocks;
 	int crc_offset;
+	int sr_bytes;
 	__u32 sum;
 
 	/* fill in segment summary checksum */
@@ -1669,9 +1643,10 @@ static void fill_in_checksums(struct nilfs_segment_info *si, __u32 crc_seed)
 
 	/* fill in super root checksum */
 	crc_offset = sizeof(nilfs.super_root->sr_sum);
+	sr_bytes = NILFS_SR_BYTES(sizeof(struct nilfs_inode));
 	sum = nilfs_crc32(crc_seed,
 			  (unsigned char *)nilfs.super_root + crc_offset,
-			  NILFS_SR_BYTES - crc_offset);
+			  sr_bytes - crc_offset);
 	nilfs.super_root->sr_sum = cpu_to_le32(sum);
 
 	/* fill in segment checksum */
